<?xml version="1.0" encoding="UTF-8"?>
<!--
Crane-ods2ublgc-obfuscated.xsl

This is a stylesheet to convert instances of spreadsheet ODS binary or
ODS XML into a genericode file suitable for processing according to the
OASIS Universal Business Language (UBL) Naming and Design Rules (NDR).

Copyright (C) - Crane Softwrights Ltd. 
              - http://www.CraneSoftwrights.com/links/training-gctk.htm

Portions copyright (C) - OASIS Open 2015. All Rights Reserved.
                       - http://www.oasis-open.org/who/intellectualproperty.php

This documentation and the URI resolution of the XSLT processor both presume
the use of the free Saxon9HE XSLT processor from http://saxon.sf.net ... if
that XSLT 2.0 processor is not being used, then the results may not be as
expected.

Input file (choose one of the following three options):

 (a) -s:{filename} - an XML file listing as the input invocation argument
       that lists the directories and files in which to find the names of
       ODS binary files to be amalgamated into a single output file (see
       below for a copy of the ubl21files.xml example file)
 (b) -it:ods-uri ods-uri={filename} - both arguments needed for an ODS
       binary file name invocation parameter
 (c) -s:{filename} - an ODS XML file as the input invocation argument

Additional invocation arguments:

 - stylesheet file:            -xsl:Crane-ods2ublgc-obfuscated.xsl
 - output genericode file:     -o:{filename}
 
Additional optional invocation parameters:

 - add row as a column value   row-number-column-name={string}

 - identification metadata:    identification-uri={filename}
   as an XML document to use in place of generated metadata; the file is
   of the form of the identification element:

     <Identification>
        <ShortName>UBLNDRSkeleton</ShortName>
        <LongName>UBL NDR skeleton genericode file</LongName>
        <Version>1</Version>
        <CanonicalUri>urn:X-CraneSoftwrights.com</CanonicalUri>
        <CanonicalVersionUri>urn:X-CraneSoftwrights.com</CanonicalVersionUri>
     </Identification>

     (note that a relative URI is relative to the stylesheet, not the data;
      use an absolute URI to avoid using a relative URI)

 - indentation of the result:  indent=yes/no (default 'yes')
 
 - indication of raw output:   raw-sheet-long-name={long name of sheet field}
   - the raw output is used when the output is not destined for the UBL NDR
   - the raw output has no key fields and all fields are marked as optional
   - the raw sheet long name, and the derived short name, are used as column
     identifiers for the sheet name value
   - set the raw sheet long name to the empty string to suppress output of the
     sheet name value

 - massaging worksheet names:  lengthen-model-name-uri={filename}
   this points to an XML document with regular expressions used to reverse
   the act of shortening worksheet names; the example in the file (copied
   below) massageModelName-2.1.xml illustrates the shortening and lengthening
   of names to sidestep the bug in Google Docs (see *)

Spreadsheet assumptions:

Each worksheet tab of the spreadsheet is assumed to be a separate model.  The 
first row of the worksheet tab is assumed to be item headings.  The order of 
the columns is not significant.  Empty rows are ignored.  A row whose entire 
text content of the concatenation of all columns is the word "END" signals 
that any subsequent rows are ignored because they are documentary.

Note that if your spreadsheet is in Microsoft Excel format it can be translated
into OpenDocument Format Spreadsheet (ODS) by one of these free approaches:

 (a) - uploading the XLS to Google Drive, opening the spreadsheet on Google,
       then using File/"Download as..." as an "OpenDocument format (.ods)" file
       unless the worksheet names are longer than 31 characters (see *)
 (b) - use the http://www.CraneSoftwrights.com/resources/index.htm#xls2ods2xml
       package after installing it in Open Office

Google Docs:

*  As of May 2015 Google Docs has a bug in that it imports worksheet
   names longer than 31 characters but truncates those names to 31 characters
   when exporting as an ODS file.  When this bug is repaired, no special
   massaging will be needed for worksheet names.


The specification for OASIS genericode is here:

    http://docs.oasis-open.org/codelist/genericode
    
The rows of the spreadsheet are converted into rows of genericode, stopping
at either the end of the spreadsheet worksheet or at the row whose entire
content is the word "END" (whichever comes first).

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE. THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

TECHNICAL NOTE:

This stylesheet has been purposely obfuscated and all comments have been
removed.  Please respect our copyright and do not attempt to reverse 
engineer the techniques involved.

- - - - - - - - - - - - - - - - - - - -

Example input XML file of document names (included as ubl21files.xml):

<directory name=".">
  <directory name="maindoc">
    <file name="UBL-ApplicationResponse-2.1.ods"/>
    <file name="UBL-AttachedDocument-2.1.ods"/>
    <file name="UBL-AwardedNotification-2.1.ods"/>
    <file name="UBL-BillOfLading-2.1.ods"/>
    <file name="UBL-CallForTenders-2.1.ods"/>
    <file name="UBL-Catalogue-2.1.ods"/>
    <file name="UBL-CatalogueDeletion-2.1.ods"/>
    <file name="UBL-CatalogueItemSpecificationUpdate-2.1.ods"/>
    <file name="UBL-CataloguePricingUpdate-2.1.ods"/>
    <file name="UBL-CatalogueRequest-2.1.ods"/>
    <file name="UBL-CertificateOfOrigin-2.1.ods"/>
    <file name="UBL-ContractAwardNotice-2.1.ods"/>
    <file name="UBL-ContractNotice-2.1.ods"/>
    <file name="UBL-CreditNote-2.1.ods"/>
    <file name="UBL-DebitNote-2.1.ods"/>
    <file name="UBL-DespatchAdvice-2.1.ods"/>
    <file name="UBL-DocumentStatus-2.1.ods"/>
    <file name="UBL-DocumentStatusRequest-2.1.ods"/>
    <file name="UBL-ExceptionCriteria-2.1.ods"/>
    <file name="UBL-ExceptionNotification-2.1.ods"/>
    <file name="UBL-Forecast-2.1.ods"/>
    <file name="UBL-ForecastRevision-2.1.ods"/>
    <file name="UBL-ForwardingInstructions-2.1.ods"/>
    <file name="UBL-FreightInvoice-2.1.ods"/>
    <file name="UBL-FulfilmentCancellation-2.1.ods"/>
    <file name="UBL-GoodsItemItinerary-2.1.ods"/>
    <file name="UBL-GuaranteeCertificate-2.1.ods"/>
    <file name="UBL-InstructionForReturns-2.1.ods"/>
    <file name="UBL-InventoryReport-2.1.ods"/>
    <file name="UBL-Invoice-2.1.ods"/>
    <file name="UBL-ItemInformationRequest-2.1.ods"/>
    <file name="UBL-Order-2.1.ods"/>
    <file name="UBL-OrderCancellation-2.1.ods"/>
    <file name="UBL-OrderChange-2.1.ods"/>
    <file name="UBL-OrderResponse-2.1.ods"/>
    <file name="UBL-OrderResponseSimple-2.1.ods"/>
    <file name="UBL-PackingList-2.1.ods"/>
    <file name="UBL-PriorInformationNotice-2.1.ods"/>
    <file name="UBL-ProductActivity-2.1.ods"/>
    <file name="UBL-Quotation-2.1.ods"/>
    <file name="UBL-ReceiptAdvice-2.1.ods"/>
    <file name="UBL-Reminder-2.1.ods"/>
    <file name="UBL-RemittanceAdvice-2.1.ods"/>
    <file name="UBL-RequestForQuotation-2.1.ods"/>
    <file name="UBL-RetailEvent-2.1.ods"/>
    <file name="UBL-SelfBilledCreditNote-2.1.ods"/>
    <file name="UBL-SelfBilledInvoice-2.1.ods"/>
    <file name="UBL-Statement-2.1.ods"/>
    <file name="UBL-StockAvailabilityReport-2.1.ods"/>
    <file name="UBL-Tender-2.1.ods"/>
    <file name="UBL-TendererQualification-2.1.ods"/>
    <file name="UBL-TendererQualificationResponse-2.1.ods"/>
    <file name="UBL-TenderReceipt-2.1.ods"/>
    <file name="UBL-TradeItemLocationProfile-2.1.ods"/>
    <file name="UBL-TransportationStatus-2.1.ods"/>
    <file name="UBL-TransportationStatusRequest-2.1.ods"/>
    <file name="UBL-TransportExecutionPlan-2.1.ods"/>
    <file name="UBL-TransportExecutionPlanRequest-2.1.ods"/>
    <file name="UBL-TransportProgressStatus-2.1.ods"/>
    <file name="UBL-TransportProgressStatusRequest-2.1.ods"/>
    <file name="UBL-TransportServiceDescription-2.1.ods"/>
    <file name="UBL-TransportServiceDescriptionRequest-2.1.ods"/>
    <file name="UBL-UnawardedNotification-2.1.ods"/>
    <file name="UBL-UtilityStatement-2.1.ods"/>
    <file name="UBL-Waybill-2.1.ods"/>
  </directory>
  <directory name="common">
    <file name="UBL-CommonLibrary-2.1.ods"/>
  </directory>
</directory>

Example input file of regular expressions to shorten or lengthen names:

<!- -
  $Id: massageModelName-2.1.xml,v 1.3 2015/05/06 14:49:29 admin Exp $
  
  Use this file to express the regular expressions that shorten model
  names in document order and lengthen model names in reverse document order.

  Use this expression in the mod/maindoc directory to establish which names
  need shortening to 31 characters or less:

ls *.ods | sed s/UBL-// | sed s/-2.1.ods// | sed -E "s/(.{1,31})(.*)/\1    \2/"

- ->
<!DOCTYPE modelNameMassage
[
<!ELEMENT modelNameMassage ( pass+ )>
<!ELEMENT pass ( shorten, lengthen )>
<!ELEMENT shorten EMPTY>
<!ELEMENT lengthen EMPTY>
<!ATTLIST shorten find CDATA #REQUIRED replace CDATA #REQUIRED>
<!ATTLIST lengthen find CDATA #REQUIRED replace CDATA #REQUIRED>
]>
<modelNameMassage>
  <pass>
    <shorten find="UBL-(.+)-2.1" replace="$1"/>
    <lengthen find="(.+)" replace="UBL-$1-2.1"/>
  </pass>
  <pass>
    <shorten find="Catalogue" replace="Ctlg"/>
    <lengthen find="Ctlg" replace="Catalogue"/>
  </pass>
  <pass>
    <shorten find="Transport([^a])" replace="Txp$1"/>
    <lengthen find="Txp" replace="Transport"/>
  </pass>
  <pass>
    <shorten find="Signature" replace="Sgnt"/>
    <lengthen find="Sgnt" replace="Signature"/>
  </pass>
  <pass>
    <shorten find="Confirmation" replace="Conf"/>
    <lengthen find="Conf" replace="Confirmation"/>
  </pass>
</modelNameMassage>


Crane-ods2ublgc.xsl $Id: Crane-ods2ublgc.xsl,v 1.7 2015/06/24 01:00:21 admin Exp $
gcExportSubset.xsl $Id: gcExportSubset.xsl,v 1.14 2015/06/24 01:00:22 admin Exp $
odsCommon.xsl $Id: odsCommon.xsl,v 1.18 2014/12/27 22:11:50 admin Exp $
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:gc="http://docs.oasis-open.org/codelist/ns/genericode/1.0/" xmlns:c="urn:X-Crane:stylesheets:gc-toolkit" xmlns:o="urn:X-Crane:stylesheets:odf-access" xmlns:office="urn:oasis:names:tc:opendocument:xmlns:office:1.0" xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0" xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0" exclude-result-prefixes="xs xsd gc c o office table text" version="2.0"><xsl:param name="row-number-column-name" as="xsd:string?" select="()"/><xsl:param name="identification-uri" as="xsd:string?" select="()"/><xsl:param name="ods-uri" as="xsd:string?" select="()"/><xsl:param name="raw-sheet-long-name" as="xsd:string?" select="()"/><xsl:param name="indent" as="xsd:string" select="'yes'"/><xsl:function name="c:I845372731" as="xsd:string?"><xsl:param name="c:I845372731" as="xsd:string?"/><xsl:variable name="c:I845372731" select="translate($c:I845372731,'\','/')"/><xsl:sequence select="concat('jar:file:',$c:I845372731,'!/content.xml')"/></xsl:function><xsl:template name="ods-uri"><xsl:variable name="c:I422686365" select="c:I845372731($ods-uri)"/><xsl:variable name="c:O281790910" as="document-node()?"><xsl:if test="not(doc-available($c:I422686365))"><xsl:message terminate="yes"><xsl:text>The given URI "</xsl:text><xsl:value-of select="replace($ods-uri,'^jar:file:(/(.:))?(.*)!/content.xml$','$2$3')"/><xsl:text>" does not appear to </xsl:text><xsl:text>point to a ZIP package with a "content.xml" file.</xsl:text></xsl:message></xsl:if><xsl:sequence select="doc($c:I422686365)"/></xsl:variable><xsl:for-each select="$c:O281790910"><xsl:call-template name="c:O211343182"><xsl:with-param name="c:O169074546" select="$c:O281790910"/></xsl:call-template></xsl:for-each></xsl:template><xsl:template match="/" priority="1"><xsl:result-document indent="{if( lower-case($indent)=('y','yes') ) then 'yes' else 'no'}"><xsl:apply-templates/></xsl:result-document></xsl:template><xsl:template match="/directory" priority="3"><xsl:variable name="c:O169074546" as="document-node()*"><xsl:for-each select="//file"><xsl:variable name="c:I140895455" select="replace( resolve-uri(string-join(ancestor-or-self::*/@name,'/'),document-uri(/)), '^file:(/(.:))?','$2')"/><xsl:choose><xsl:when test="doc-available($c:I140895455)"><xsl:choose><xsl:when test="not(doc($c:I140895455)/ (office:document | office:document-content)/ office:body/office:spreadsheet/table:table)"><xsl:message terminate="yes"><xsl:text>The given URI "</xsl:text><xsl:value-of select="$c:I140895455"/><xsl:text>" does not appear to </xsl:text><xsl:text>point to an XML ODS file.</xsl:text></xsl:message></xsl:when><xsl:otherwise><xsl:sequence select="doc($c:I140895455)"/></xsl:otherwise></xsl:choose></xsl:when><xsl:otherwise><xsl:variable name="c:I120767533" select="c:I845372731($c:I140895455)"/><xsl:choose><xsl:when test="not(doc-available($c:I120767533))"><xsl:message terminate="yes"><xsl:text>The given URI "</xsl:text><xsl:value-of select="$c:I140895455"/><xsl:text>" does not appear to </xsl:text><xsl:text>point to a binary ODF file.</xsl:text></xsl:message></xsl:when><xsl:when test="not(doc($c:I120767533)/ (office:document | office:document-content)/ office:body/office:spreadsheet/table:table)"><xsl:message terminate="yes"><xsl:text>The given URI "</xsl:text><xsl:value-of select="$c:I140895455"/><xsl:text>" does not appear to </xsl:text><xsl:text>point to a binary ODS file.</xsl:text></xsl:message></xsl:when><xsl:otherwise><xsl:sequence select="doc($c:I120767533)"/></xsl:otherwise></xsl:choose></xsl:otherwise></xsl:choose></xsl:for-each></xsl:variable><xsl:if test="count(//file) != count($c:O169074546)"><xsl:message terminate="yes"/></xsl:if><xsl:call-template name="c:O211343182"><xsl:with-param name="c:O169074546" select="$c:O169074546"/></xsl:call-template></xsl:template><xsl:template match="/office:document | /office:document-content" priority="2" name="c:O211343182"><xsl:param name="c:O169074546" select=".." as="document-node()+"/><xsl:variable name="c:I105671591" as="element(Column)*"><xsl:for-each-group group-by="." select="($c:O169074546/*/office:body/ office:spreadsheet/table:table//table:table-row)[1]/table:table-cell/text:p"><xsl:variable name="c:I93930303" select="normalize-space(current-grouping-key())"/><xsl:variable name="c:I84537273" select="replace($c:I93930303,'\W+','')"/><Column Id="{$c:I84537273}" Use="{if( $c:I84537273 = 'DictionaryEntryName' and                         empty( $raw-sheet-long-name ) )                      then 'required' else 'optional'}"><ShortName><xsl:value-of select="$c:I84537273"/></ShortName><LongName><xsl:value-of select="$c:I93930303"/></LongName><Data Type="string"/></Column></xsl:for-each-group></xsl:variable><xsl:if test="empty($raw-sheet-long-name)"><xsl:variable name="c:O76852066"><xsl:if test="($c:I105671591/@Id,$row-number-column-name)= 'ModelName'"><xsl:text>Cannot have a column named "ModelName" as this is </xsl:text><xsl:text>reserved to be defined by the worksheet name.
</xsl:text></xsl:if><xsl:if test="not($c:I105671591/@Id='DictionaryEntryName')"><xsl:text>Must have a column named "DictionaryEntryName" as </xsl:text><xsl:text>this is the key column for the genericode file.
</xsl:text></xsl:if><xsl:if test="$c:I105671591/@Id=$row-number-column-name"><xsl:text>Cannot have a column with the same name as the </xsl:text><xsl:text>column designated to store the row number.
</xsl:text></xsl:if></xsl:variable><xsl:if test="normalize-space($c:O76852066)"><xsl:message terminate="yes" select="$c:O76852066"/></xsl:if></xsl:if><gc:CodeList><xsl:call-template name="c:I70447727"/><ColumnSet><xsl:choose><xsl:when test="empty($raw-sheet-long-name)"><Column Id="ModelName" Use="required"><ShortName>ModelName</ShortName><LongName>Model Name</LongName><Data Type="string"/></Column><xsl:for-each select="$row-number-column-name"><Column Id="{translate(normalize-space(.),' ','')}" Use="required"><ShortName><xsl:value-of select="translate(normalize-space(.),' ','')"/></ShortName><LongName><xsl:value-of select="."/></LongName><Data Type="integer"/></Column></xsl:for-each><xsl:copy-of select="$c:I105671591"/><Key Id="key"><ShortName>Key</ShortName><ColumnRef Ref="DictionaryEntryName"/></Key></xsl:when><xsl:when test="string($raw-sheet-long-name)"><Column Id="{$worksheetIdentifier}" Use="optional"><ShortName><xsl:value-of select="$worksheetIdentifier"/></ShortName><LongName><xsl:value-of select="$raw-sheet-long-name"/></LongName><Data Type="string"/></Column><xsl:copy-of select="$c:I105671591"/></xsl:when><xsl:otherwise><xsl:copy-of select="$c:I105671591"/></xsl:otherwise></xsl:choose></ColumnSet><SimpleCodeList><xsl:call-template name="c:I65028671"><xsl:with-param name="c:O169074546" select="$c:O169074546"/></xsl:call-template></SimpleCodeList></gc:CodeList></xsl:template><xsl:template match="/*" priority="1"><xsl:apply-templates mode="c:O60383766" select="."><xsl:with-param name="c:O56358182" tunnel="yes" as="text()*"><xsl:text>This filter does not support an XML instance </xsl:text><xsl:value-of select="concat('starting with {',namespace-uri(*),'}', name(*))"/></xsl:with-param></xsl:apply-templates></xsl:template><xsl:template name="c:I70447727"><xsl:choose><xsl:when test="exists($identification-uri)"><xsl:choose><xsl:when test="not(doc-available($identification-uri))"><xsl:message terminate="yes"><xsl:text>The file at parameter identification-uri= </xsl:text><xsl:text>is not found or is not an XML file.</xsl:text></xsl:message></xsl:when><xsl:when test="not(doc($identification-uri)/Identification)"><xsl:message terminate="yes"><xsl:text>The file at parameter identification-uri= </xsl:text><xsl:text>does not have "{}Identification" as the </xsl:text><xsl:text>document element.</xsl:text></xsl:message></xsl:when><xsl:otherwise><xsl:copy-of select="doc($identification-uri)/Identification"/></xsl:otherwise></xsl:choose></xsl:when><xsl:otherwise><xsl:message select="'Using built-in placebo Identification metadata'"/><Identification><ShortName>UBLNDRSkeleton</ShortName><LongName>UBL NDR skeleton genericode file</LongName><Version>1</Version><CanonicalUri>urn:X-CraneSoftwrights.com</CanonicalUri><CanonicalVersionUri>urn:X-CraneSoftwrights.com</CanonicalVersionUri></Identification></xsl:otherwise></xsl:choose></xsl:template><xsl:param name="worksheetIdentifier" as="xsd:string?" select="if( exists( $raw-sheet-long-name ) ) then if( string( $raw-sheet-long-name ) ) then replace($raw-sheet-long-name,'\W+','') else () else 'ModelName'"/><xsl:param name="lengthen-model-name-uri" as="xsd:string?"/><xsl:param name="c:I52835795" as="document-node()?" select="if( doc-available( $lengthen-model-name-uri ) ) then for $d in doc( $lengthen-model-name-uri ) return if ( $d/modelNameMassage ) then $d else () else ()"/><xsl:output indent="yes"/><xsl:template match="/"><xsl:choose><xsl:when test="not(office:document or office:document-content)"><xsl:apply-templates mode="c:O60383766" select="node()"><xsl:with-param name="c:O56358182" tunnel="yes" as="text()*"><xsl:text>This filter does not support an instance </xsl:text><xsl:value-of select="concat('starting with {',namespace-uri(*),'}', name(*))"/></xsl:with-param></xsl:apply-templates></xsl:when><xsl:otherwise><xsl:call-template name="c:I65028671"><xsl:with-param name="c:O169074546" select="/"/></xsl:call-template></xsl:otherwise></xsl:choose></xsl:template><xsl:template name="c:I65028671"><xsl:param name="c:O169074546" as="document-node()*"/><xsl:for-each select="$c:O169074546/*/office:body/office:spreadsheet/ table:table"><xsl:variable name="c:I49727807" select="c:I46965151(@table:name)"/><xsl:variable name="c:I44493301" as="element()*"><xsl:for-each select="(.//table:table-row)[1]"><xsl:variable name="c:O42268636" select="."/><xsl:for-each select="1 to xsd:integer( o:I845372731(table:table-cell[last()])[last()] )"><guess><xsl:value-of select="replace(translate( $c:O42268636/table:table-cell[o:I845372731(.)=current()]/o:I422686365(.), ':()/''',''),'\s+','')"/></guess></xsl:for-each></xsl:for-each></xsl:variable><xsl:variable name="c:O40255844" select="( (.//table:table-row)[normalize-space( string-join(table:table-cell/o:I422686365(.),''))='END']/ preceding::table:table-row[1], (.//table:table-row)[last()] )[1]"/><xsl:for-each select="(.//table:table-row)[position()&gt;1] [ . is $c:O40255844 or . &lt;&lt; $c:O40255844 ] [string-join(table:table-cell/o:I422686365(.),'')!='']"><Row><xsl:if test="not($row-number-column-name)"><xsl:comment select="position()+1"/></xsl:if><xsl:if test="lower-case($indent)=('y','yes')"><xsl:text>
         </xsl:text></xsl:if><xsl:if test="$worksheetIdentifier"><Value ColumnRef="{$worksheetIdentifier}"><SimpleValue><xsl:value-of select="$c:I49727807"/></SimpleValue></Value></xsl:if><xsl:if test="$row-number-column-name"><Value ColumnRef="{translate(normalize-space($row-number-column-name),' ','')}"><SimpleValue><xsl:value-of select="position()+1"/></SimpleValue></Value></xsl:if><xsl:variable name="c:O42268636" select="."/><xsl:for-each select="1 to xsd:integer( o:I845372731(table:table-cell[last()])[last()] )"><xsl:variable name="c:I38426033" select="."/><xsl:for-each select="$c:O42268636/table:table-cell [o:I845372731(.)=$c:I38426033]/ o:I422686365(.)[normalize-space(.)]"><Value ColumnRef="{$c:I44493301[$c:I38426033]}"><SimpleValue><xsl:value-of select="."/></SimpleValue></Value></xsl:for-each></xsl:for-each></Row></xsl:for-each></xsl:for-each></xsl:template><xsl:function name="c:I46965151" as="xsd:string?"><xsl:param name="c:O36755336" as="xsd:string?"/><xsl:sequence select="if( exists($c:O36755336) ) then c:I46965151($c:O36755336,$c:I52835795/*/pass[last()]) else ()"/></xsl:function><xsl:function name="c:I46965151" as="xsd:string?"><xsl:param name="c:O36755336" as="xsd:string?"/><xsl:param name="c:I35223863" as="element(pass)?"/><xsl:sequence select="if( $c:I35223863 ) then c:I46965151( replace( $c:O36755336, $c:I35223863/lengthen/@find, $c:I35223863/lengthen/@replace ), $c:I35223863/preceding-sibling::pass[1] ) else $c:O36755336"/></xsl:function><xsl:variable name="odsTemplate" as="document-node()"><xsl:document><office:document/></xsl:document></xsl:variable><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O211343182" as="element(table:table)*"><xsl:param name="o:O281790910" as="element()"/><xsl:sequence select="$o:O281790910/*/office:body/office:spreadsheet/table:table"/></xsl:function><xsl:key xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O169074546" match="style:style" use="@style:name"/><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I120767533" as="xsd:string"><xsl:param name="o:I140895455" as="attribute()*"/><xsl:sequence select="if( not( $o:I140895455 ) ) then '' else if( starts-with($o:I140895455,'Crane') ) then $o:I140895455 else o:I120767533(key('o:O169074546',$o:I140895455,root($o:I140895455))/ @style:parent-style-name)"/></xsl:function><xsl:key xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I105671591" match="table:table-cell[ o:I120767533(preceding-sibling::table:table-cell[2]/@table:style-name)]" use="substring-after( o:I120767533(preceding-sibling::table:table-cell[2]/@table:style-name), 'CraneLabel')"/><xsl:key xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I93930303" match="table:table-cell[ o:I120767533(preceding-sibling::table:table-cell[2]/@table:style-name)]" use="1"/><xsl:key xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I84537273" match="table:table-cell[o:I120767533(@table:style-name)]" use="o:I120767533(@table:style-name)"/><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I65028671" as="node()?"><xsl:param name="o:O76852066" as="xsd:string"/><xsl:param name="o:I70447727" as="node()"/><xsl:sequence select="key('o:I105671591',$o:O76852066,$o:I70447727)"/></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I422686365" as="xsd:string?"><xsl:param name="o:O60383766" as="element(table:table-cell)?"/><xsl:sequence select="string-join($o:O60383766/text:p/string(.),' ')"/></xsl:function><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O56358182"><xsl:analyze-string select="." regex=".+"><xsl:matching-substring><text:p><xsl:value-of select="."/></text:p></xsl:matching-substring></xsl:analyze-string></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I44493301"><xsl:param name="o:I52835795" as="document-node()" required="yes"/><xsl:param name="o:I49727807" tunnel="yes" as="xsd:boolean" select="false()"/><xsl:param name="o:I46965151" tunnel="yes" as="xsd:boolean" select="false()"/><xsl:copy><xsl:if test="not($o:I49727807)"><xsl:apply-templates select="@*"/></xsl:if><xsl:choose><xsl:when test="$o:I52835795"><xsl:variable name="o:O42268636" as="document-node()"><xsl:document><xsl:apply-templates select="$o:I52835795/node()" mode="o:I44493301"/></xsl:document></xsl:variable><xsl:for-each select="$o:O42268636"><xsl:call-template name="o:O56358182"/></xsl:for-each></xsl:when><xsl:when test="$o:I46965151"/><xsl:otherwise><xsl:apply-templates/></xsl:otherwise></xsl:choose></xsl:copy></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" mode="o:I44493301" match="*"><xsl:param name="o:O40255844" tunnel="yes" as="xsd:string*" select="()"/><xsl:text/>&lt;<xsl:value-of select="name(.)"/><xsl:for-each select="namespace::*[not(.= (../namespace::xml,$o:O40255844))]"><xsl:if test="not(name(.)=../../namespace::*/name(.))"><xsl:value-of select="concat(' xmlns',if(name()) then ':' else '', name(.),'=&#34;',.,'&#34;')"/></xsl:if></xsl:for-each><xsl:for-each select="@*"><xsl:choose><xsl:when test="contains(.,'&#34;')"><xsl:value-of select="concat(&#34; &#34;,name(.),&#34;='&#34;,.,&#34;'&#34;)"/></xsl:when><xsl:otherwise><xsl:value-of select="concat(' ',name(.),'=&#34;',.,'&#34;')"/></xsl:otherwise></xsl:choose></xsl:for-each><xsl:choose><xsl:when test="node()"><xsl:text>&gt;</xsl:text><xsl:apply-templates mode="o:I44493301" select="node()"/><xsl:text/>&lt;/<xsl:value-of select="name(.)"/>&gt;<xsl:text/></xsl:when><xsl:otherwise><xsl:text>/&gt;</xsl:text></xsl:otherwise></xsl:choose></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" mode="o:I44493301" match="text()"><xsl:value-of select="."/></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" mode="o:I44493301" match="comment()"><xsl:value-of select="concat('&lt;!--',.,'--&gt;')"/></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" mode="o:I44493301" match="processing-instruction()"><xsl:value-of select="concat('&lt;?',name(.),' ',.,'?&gt;')"/></xsl:template><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" match="@*|node()" mode="#default o:I38426033"><xsl:copy><xsl:apply-templates select="@*,node()" mode="#default"/></xsl:copy></xsl:template><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I35223863" as="xsd:anyAtomicType*"><xsl:param name="o:O36755336" as="xsd:anyAtomicType*"/><xsl:perform-sort select="$o:O36755336"><xsl:sort/></xsl:perform-sort></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I32514335" as="node()*"><xsl:param name="o:I33814909" as="node()*"/><xsl:perform-sort select="$o:I33814909"><xsl:sort/></xsl:perform-sort></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I845372731" as="xsd:double+"><xsl:param name="o:O60383766" as="node()"/><xsl:for-each select="$o:O60383766[self::table:table-cell]"><xsl:variable name="o:I31310101" select="count(preceding-sibling::table:table-cell) + 1 + sum(preceding-sibling::table:table-cell/@table:number-columns-spanned) + sum(preceding-sibling::table:table-cell/@table:number-columns-repeated) - count(preceding-sibling::table:table-cell[@table:number-columns-spanned]) - count(preceding-sibling::table:table-cell[@table:number-columns-repeated])"/><xsl:sequence select="xsd:integer($o:I31310101) to xsd:integer( $o:I31310101 - 1 + ( if( @table:number-columns-repeated ) then @table:number-columns-repeated else 1 ) )"/></xsl:for-each><xsl:for-each select="$o:O60383766[self::table:table-column]"><xsl:variable name="start" select="count(preceding-sibling::table:table-column) + 1 + sum(preceding-sibling::table:table-column/@table:number-columns-repeated) - count(preceding-sibling::table:table-column[@table:number-columns-repeated])"/><xsl:sequence select="xsd:integer($start) to xsd:integer( $start - 1 + ( if( @table:number-columns-repeated ) then @table:number-columns-repeated else 1 ) )"/></xsl:for-each></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I30191883" as="xsd:double+"><xsl:param name="o:I30191883" as="element(table:table-row)"/><xsl:for-each select="$o:I30191883"><xsl:variable name="start" select="count(preceding-sibling::table:table-row) + 1 + sum(preceding-sibling::table:table-row/@table:number-rows-repeated) - count(preceding-sibling::table:table-row[@table:number-rows-repeated])"/><xsl:sequence select="xsd:integer($start) to xsd:integer( $start - 1 + ( if( @table:number-rows-repeated ) then @table:number-rows-repeated else 1 ) )"/></xsl:for-each></xsl:function><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I29150783" mode="o:I29150783" match="table:table-cell"><xsl:param name="o:I52835795" as="node()?" required="yes"/><xsl:param name="o:I49727807" tunnel="yes" as="xsd:boolean" select="false()"/><xsl:param name="o:I46965151" tunnel="yes" as="xsd:boolean" select="false()"/><xsl:copy><xsl:if test="not($o:I49727807)"><xsl:apply-templates select="@*"/></xsl:if><xsl:choose><xsl:when test="string($o:I52835795)"><xsl:if test="not($o:I46965151)"><xsl:apply-templates select="* except text:*"/></xsl:if><xsl:for-each select="$o:I52835795"><xsl:call-template name="o:O56358182"/></xsl:for-each></xsl:when><xsl:when test="$o:I46965151"/><xsl:otherwise><xsl:apply-templates/></xsl:otherwise></xsl:choose></xsl:copy></xsl:template><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O27270088" as="xsd:integer"><xsl:param name="o:I28179091" as="xsd:string"/><xsl:analyze-string select="$o:I28179091" regex="\s*(([A-Z])?([A-Z]))?([A-Z])\s*"><xsl:matching-substring><xsl:sequence select="o:I26417897(regex-group(2))*26*26 + o:I26417897(regex-group(3))*26 + o:I26417897(regex-group(4))"/></xsl:matching-substring></xsl:analyze-string></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I26417897" as="xsd:integer"><xsl:param name="o:I28179091" as="xsd:string"/><xsl:sequence select="if($o:I28179091='') then 0 else string-to-codepoints($o:I28179091) - string-to-codepoints('A') + 1"/></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I24863903" as="xsd:string"><xsl:param name="o:I25617355" as="xsd:integer"/><xsl:variable name="o:O24153506" select="( $o:I25617355 - 1 ) idiv (27*26)"/><xsl:variable name="o:I23482575" select="(($o:I25617355 - 1 + $o:O24153506*26) mod (27*26) idiv 26)"/><xsl:variable name="o:I22847911" select="( $o:I25617355 - 1 ) mod 26 + 1"/><xsl:sequence select="concat( o:O22246650( $o:O24153506 ), o:O22246650( $o:I23482575 ), o:O22246650( $o:I22847911 ) )"/></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O22246650" as="xsd:string"><xsl:param name="o:I21676223" as="xsd:integer"/><xsl:sequence select="if($o:I21676223=0) then '' else codepoints-to-string( $o:I21676223 + string-to-codepoints('A') - 1 )"/></xsl:function><xsl:template xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:O21134318"><xsl:for-each select="1 to 1024"><xsl:variable name="o:I20618847" select="."/><xsl:variable name="o:I28179091" select="o:I24863903($o:I20618847)"/><xsl:variable name="o:O20127922" select="o:O27270088($o:I28179091)"/><xsl:message select="concat( $o:I20618847,'=',$o:I28179091,'=',$o:O20127922 )"/><xsl:if test="$o:I20618847 != $o:O20127922"><xsl:message terminate="yes" select="'o:O19659830'"/></xsl:if></xsl:for-each></xsl:template><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I17986653" as="text()*"><xsl:param name="o:O19213016" as="xsd:anyAtomicType?"/><xsl:param name="o:O18786060" as="xsd:integer"/><xsl:param name="o:O18377668" as="xsd:integer"/><xsl:analyze-string select="$o:O19213016" regex="(\$?)([A-Z])+(\$?)(\d+)"><xsl:matching-substring><xsl:value-of select="concat(regex-group(1), o:I24863903(o:O27270088(regex-group(2)) + $o:O18786060), regex-group(3), xsd:integer(regex-group(4)) + $o:O18377668 )"/></xsl:matching-substring><xsl:non-matching-substring><xsl:value-of select="."/></xsl:non-matching-substring></xsl:analyze-string></xsl:function><xsl:function xmlns:style="urn:oasis:names:tc:opendocument:xmlns:style:1.0" name="o:I17611931" as="xsd:string"><xsl:param name="o:O60383766" as="element(table:table-cell)"/><xsl:for-each select="$o:O60383766"><xsl:sequence select="concat(ancestor::table:table[1]/@table:name,'.', o:I24863903(xsd:integer(o:I845372731(.))), o:I30191883(..))"/></xsl:for-each></xsl:function></xsl:stylesheet>